@@ -13,6 +13,7 @@ import logging
 import time
 import os
 from functools import wraps
+from typing import Iterable, List, Generator, Tuple
 
 logger = logging.getLogger(__name__)
 
@@ -32,18 +33,18 @@ def log_cost_time(name=None, level=logging.INFO):
 
 
 class LogCostContext(object):
-    def __init__(self, name, level=logging.INFO):
+    def __init__(self, name, level=logging.INFO, star_num=0):
         self.name = name
         self.level = level
+        self.star_num = star_num
 
     def __enter__(self):
-        logger.log(msg=f"{self.name} starts", level=self.level)
+        logger.log(msg="*" * self.star_num + f"{self.name} starts" + "*" * self.star_num, level=self.level)
         self.st = time.time()
 
     def __exit__(self, type, value, traceback):
         cost = time.time() - self.st
-        logger.log(msg=f"{self.name} ends, cost:{cost:4.3f} seconds", level=self.level)
-
+        logger.log(msg="*" * self.star_num +f"{self.name} ends, cost:{cost:4.3f} seconds"+"*" * self.star_num, level=self.level)
 
 # 执行函数时输出函数的参数以及返回值
 def log_function_info(input_level=logging.DEBUG, result_level=logging.DEBUG,
@@ -102,3 +103,24 @@ def discard_kwarg(func):
         return func(*args, **kwargs)
 
     return wrap
+
+
+# adapt function with single elements
+def adapt_single(ele_name):
+    def wrapper(func):
+        @wraps(func)
+        def wrapped(*args, **kwargs):
+            if ele_name in kwargs:
+                is_single = not isinstance(kwargs[ele_name], (List, Generator, Tuple))
+            else:
+                is_single = False
+            if is_single:
+                kwargs[ele_name] = [kwargs[ele_name]]
+            rs = func(*args, **kwargs)
+            if is_single:
+                rs = rs[0]
+            return rs
+
+        return wrapped
+
+    return wrapper
